package com.displaynone.acss.ui.profile

import android.os.Bundle
import android.util.Log
import android.view.View
import android.widget.Toast
import androidx.activity.OnBackPressedCallback
import androidx.fragment.app.Fragment
import androidx.fragment.app.viewModels
import androidx.navigation.fragment.findNavController
import androidx.recyclerview.widget.LinearLayoutManager
import com.bumptech.glide.Glide
import com.displaynone.acss.R
import com.displaynone.acss.components.acs.models.visit.VisitAdapter
import com.displaynone.acss.components.auth.models.user.UserServiceST
import com.displaynone.acss.components.auth.models.user.repository.dto.UserDTO
import com.displaynone.acss.databinding.FragmentProfileBinding
import com.displaynone.acss.ui.init.InitFragmentViewModel
import com.displaynone.acss.ui.profile.ProfileViewModel.Action
import com.displaynone.acss.ui.scan.QrScanDestination
import com.displaynone.acss.util.collectWithLifecycle
import com.displaynone.acss.util.navigateTo

class ProfileFragment : Fragment(R.layout.fragment_profile) {
    private var _binding: FragmentProfileBinding? = null
    private val binding: FragmentProfileBinding get() = _binding!!

    private val viewModel: ProfileViewModel by viewModels()
    private val adapter = VisitAdapter()

    override fun onViewCreated(view: View, savedInstanceState: Bundle?) {
        super.onViewCreated(view, savedInstanceState)
        _binding = FragmentProfileBinding.bind(view)

        checkForAdmin()

        binding.swipeRefresh.setOnRefreshListener {
            if (getIsMe()) refreshData() else showData(getUserDto()!!);
            binding.swipeRefresh.isRefreshing = false
        }

        binding.logout.setOnClickListener {
            logout()
        }

        binding.recyclerViewLogs.adapter = adapter

        if (getIsMe()) {
            refreshData()

            viewModel.visitListState.collectWithLifecycle(this) { data -> adapter.submitData(data) }
//            waitForQRScanResult()
        } else {
            showData(getUserDto()!!)
            binding.backToSearch.visibility = View.VISIBLE

            binding.backToSearch.setOnClickListener {
                navigateTo(view, R.id.action_profileFragment_to_adminFragment)
            }
            Log.d("ProfileFragment", "set login")
            viewModel.visitListStateFromLogin.collectWithLifecycle(this) { data ->
                adapter.submitData(data)
                Log.d("ProfileFragment", "adapter submitted data")
            }
            hideButtons()
        }
        subscribe()
        binding.recyclerViewLogs.layoutManager = LinearLayoutManager(requireContext())
    }

    private fun checkForAdmin() {
        val userDTO = UserServiceST.getInstance().getUserDTO() ?: return

        Log.d("1234", userDTO.isACSBlocked.toString())
        binding.rightsUsingSmartphone.text = if (userDTO.isACSBlocked) "Пропуск недействителен" else "Пропуск действителен"
    }

    private fun hideButtons() {
        binding.logout.visibility = View.GONE
        binding.changeRights.visibility = View.VISIBLE
    }

    private fun showMyData(userDTO: UserDTO) {
        binding.fio.text = userDTO.name
        binding.position.text = userDTO.position
        setAvatar(userDTO.photo)
    }
    private fun updateRights(isACSBlocked: Boolean){
        binding.changeRights.text = if(isACSBlocked) "Разблокировать пользователя" else "Заблокировать пользователя"
    }

    private fun showData(userDTO: UserDTO) {
        binding.fio.text = userDTO.name
        binding.position.text = userDTO.position
        updateRights(userDTO.isACSBlocked)
        viewModel.setLogin(login1 = userDTO.login)
        binding.changeRights.setOnClickListener {
            viewModel.changeRights(userDTO.login, !userDTO.isACSBlocked)
        }
        viewModel.state.collectWithLifecycle(this){ state ->
            if (state is ProfileViewModel.State.Change){
                updateRights(state.item)
                userDTO.isACSBlocked = state.item
            }
        }
        Log.d("ProfileFragment", userDTO.login)
        setAvatar(userDTO.photo)
        checkForAdmin()
    }

    private fun refreshData() {
        viewModel.getInfo()
        subscribeToGetData()
    }

    private fun subscribe() {
        viewModel.action.collectWithLifecycle(this) { action ->
            if (action is Action.GoToAuth) {
                view?.let { navigateTo(it, R.id.action_profileFragment_to_authFragment) }
                    ?: throw IllegalStateException("View is null")
            }
            if (action is Action.GoToScan) {
                view?.let { navigateTo(it, R.id.action_profileFragment_to_qrResultFragment) }
                    ?: throw IllegalStateException("View is null")
            }
        }
    }

    private fun getUserDto(): UserDTO? {
        return arguments?.getSerializable("user") as? UserDTO
    }

    private fun getIsMe(): Boolean {
        return arguments?.getBoolean("isMe", true) ?: true
    }

//    private fun waitForQRScanResult() {
//        Log.d("1111", "122ee")
//        requireActivity().onBackPressedDispatcher.addCallback(
//            viewLifecycleOwner,
//            object : OnBackPressedCallback(true) {
//                override fun handleOnBackPressed() {
//
//                }
//            })
//
//        findNavController().currentBackStackEntry?.savedStateHandle?.getLiveData<Bundle>(
//            QrScanDestination.REQUEST_KEY
//        )?.observe(viewLifecycleOwner) { bundle ->
//            val qrCode = bundle.getString("key_qr")
//            Log.d("1111", qrCode.toString())
//
//            if (!qrCode.isNullOrEmpty()) view?.let {
//                val newBundle = Bundle().apply { putString("qrCode", qrCode) }
//                navigateTo(it, R.id.action_profileFragment_to_qrResultFragment, newBundle)
//            }
//        }
//    }

    private fun logout() {
        viewModel.logout()
        viewModel.openAuth()
        Toast.makeText(activity, "LOGOUT", Toast.LENGTH_SHORT).show()
    }

    private fun subscribeToGetData() {
        viewModel.state.collectWithLifecycle(this) { state ->
            if (state is ProfileViewModel.State.Show) {
                val userDto: UserDTO = state.item
                showMyData(userDto)
                checkForAdmin()
            }
        }
    }

    private fun setAvatar(photo: String) {
        Glide.with(requireContext())
            .load(photo)
            .placeholder(R.drawable.ic_photo)
            .error(R.drawable.ic_back)
            .into(binding.avatar)
    }

    override fun onDestroyView() {
        super.onDestroyView()
        _binding = null
    }
}